from __future__ import division, absolute_import, print_function

__all__ = ['Xcode', 'XcodeProject', 'uuids_from_binary', 'XcodeBuildProduct', 'XcodeBuildArguments']

import logging
import re

from pathlib2 import Path
from memoize import mproperty
import six

from xcute.runner import *
from xcute.semantic_version import *


class Xcode(object):
    _all_xcodes = None
    _default_xcode = None

    @classmethod
    def default(cls):
        if not Xcode._default_xcode:
            Xcode.find_all()
        return Xcode._default_xcode

    @classmethod
    def with_version(cls, version):
        if isinstance(version, six.string_types):
            version = SemanticVersion.string(version)
        if isinstance(version, int):
            version = SemanticVersion(major=version, minor=0)
        return Xcode.find_all()[version] if version in Xcode.find_all() else None

    @classmethod
    def find_all(cls):
        if Xcode._all_xcodes is None:
            output = runner.check_run('/usr/bin/mdfind \'kMDItemCFBundleIdentifier="com.apple.dt.Xcode" and kMDItemContentType="com.apple.application-bundle"\'')
            xcodes = [Xcode(Path(path)) for path in output.strip().split("\n")]
            Xcode._all_xcodes = dict([(xcode.version, xcode) for xcode in xcodes])
            default_developer_dir_path = Path(runner.check_run(['xcode-select', '-p']).strip())
            Xcode._default_xcode = [xcode for version, xcode in Xcode._all_xcodes.items() if xcode.developer_dir_path == default_developer_dir_path][0]
            Xcode._default_xcode.is_default = True
        return Xcode._all_xcodes

    def __init__(self, path):
        self.path = path
        self.is_default = False
        self.developer_dir_path = self.path / 'Contents/Developer'

    @mproperty
    def version(self):
        output = self.check_call(['xcodebuild', '-version'], env={'DEVELOPER_DIR': str(self.developer_dir_path)})
        match = re.match(r'^Xcode (?P<version>.+)\nBuild version (?P<build>.+)', output)
        return SemanticVersion.string(match.groupdict()['version'])


    # noinspection PyMethodMayBeStatic
    def call(self, command, **kwargs):
        command = runner.convert_args(command)
        command = ['/usr/bin/xcrun'] + command

        if not self.is_default:
            env = dict()
            env['DEVELOPER_DIR'] = str(self.developer_dir_path)
            if 'env' in kwargs:
                env.update(kwargs['env'])
            kwargs['env'] = env

        result = runner.run(command, **kwargs)
        return result

    # noinspection PyMethodMayBeStatic
    def check_call(self, command, **kwargs):
        kwargs['check'] = True
        result = self.call(command, **kwargs)
        return result.stdout

    def __repr__(self):
        return '{} ({})'.format(self.path, self.version)


########################################################################################################################

class XcodeProject(object):
    def __init__(self, xcode, path):
        assert xcode
        assert path

        self.xcode = xcode
        self.path = path
        self.identifier = None

    @property
    def targets(self):
        return self.info[0]

    @property
    def configurations(self):
        return self.info[1]

    @property
    def default_configuration(self):
        return self.info[3]

    @property
    def scheme_names(self):
        return self.info[2]

    @mproperty
    def schemes(self):
        return [Scheme(self, scheme_name) for scheme_name in self.scheme_names]

    def scheme_named(self, name):
        return [scheme for scheme in self.schemes if scheme.name == name][0]

    @mproperty
    def info(self):
        arguments = XcodeBuildArguments()
        arguments.project = self.path
        output = self.check_call(subcommand='-list', cache_key=self.identifier)
        targets, configurations, schemes, default_configuration = _parse_info(output)
        return targets, configurations, schemes, default_configuration

    def build_settings(self, arguments):
        # type: (XcodeBuildArguments) -> dict()
        output = self.check_call(subcommand='-showBuildSettings', arguments=arguments, cache_key=self.identifier)
        return _parse_build_settings(output)

    def build(self, arguments):
        # type: (XcodeBuildArguments) -> dict()
        try:
            self.check_call(subcommand='build', arguments=arguments)
        except CalledProcessError as e:
            logging.error('<err>Error</err>: Failed to build - result code <echo>{}</echo>'.format(e.returncode))
            logging.error('Command: <echo>{}</echo>'.format(e.cmd))
            logging.error(e.output)
            exit(e.returncode)

        build_settings = self.build_settings(arguments=arguments)
        scheme = self.scheme_named(arguments.scheme)
        assert scheme
        build_settings = build_settings[scheme.framework_target.name]
        assert build_settings
        return XcodeBuildProduct.build_settings(build_settings)

    def check_call(self, subcommand, arguments=None, **kwargs):
        # type: (str, XcodeBuildArguments) -> [str]
        assert not arguments or isinstance(arguments, XcodeBuildArguments)
        arguments = arguments.to_list() if arguments else []
        command = ['xcodebuild', '-project', self.path] + arguments + [subcommand]
        return self.xcode.check_call(command, **kwargs)

    def __repr__(self):
        return 'XcodeProject(\'{}\')'.format(self.path)

########################################################################################################################

class Scheme(object):
    def __init__(self, project, name):
        self.project = project
        self.name = name

    def __repr__(self):
        return 'Scheme("{}")'.format(self.name)


    @mproperty
    def targets(self):
        arguments = XcodeBuildArguments(scheme=self.name)
        build_settings = self.project.build_settings(arguments=arguments)
        targets = [Target(self.project, self, target_name) for target_name in build_settings.keys()]
        return targets

    @mproperty
    def framework_target(self):
        targets = [target for target in self.targets if target.product_is_framework]
        return targets[0] if targets else None


########################################################################################################################

class Target(object):
    def __init__(self, project, scheme, name):
        assert isinstance(project, XcodeProject)
        assert isinstance(scheme, Scheme)
        assert isinstance(name, six.string_types)
        self.project = project
        self.scheme = scheme
        self.name = name

    def __repr__(self):
        return 'Target("{}")'.format(self.name)

    @mproperty
    def build_settings(self):
        arguments = XcodeBuildArguments(scheme=self.scheme.name)
        build_settings = self.project.build_settings(arguments=arguments)
        return build_settings[self.name]

    @property
    def supported_platform_names(self):
        return self.build_settings.get('SUPPORTED_PLATFORMS', '').split(' ')

    @property
    def package_type(self):
        return self.build_settings.get('PACKAGE_TYPE', None)

    @property
    def product_is_framework(self):
        return self.package_type == 'com.apple.package-type.wrapper.framework'


########################################################################################################################

class XcodeBuildArguments(object):
    def __init__(self, scheme=None, target=None, configuration=None, sdk=None, toolchain=None, jobs=None, derived_data_path=None, arguments=None):
        self.scheme = scheme
        self.target = target
        self.configuration = configuration
        self.sdk = sdk
        self.toolchain = toolchain
        self.jobs = jobs
        self.derived_data_path = derived_data_path
        self.arguments = {
            'ONLY_ACTIVE_ARCH': 'NO',
            'BITCODE_GENERATION_MODE': 'bitcode',
            'CODE_SIGNING_REQUIRED': 'NO',
            'CODE_SIGN_IDENTITY': '',
            'CARTHAGE': 'YES',
        }

        if arguments:
            self.arguments.update(arguments)

    def __repr__(self):
        return 'XcodeBuildArguments({})'.format(self.__dict__)

    def to_list(self):
        # type: () -> [Any]
        parts = []
        parts += ['-scheme', self.scheme] if self.scheme else []
        parts += ['-target', self.target] if self.target else []
        parts += ['-configuration', self.configuration] if self.configuration else []
        parts += ['-sdk', self.sdk] if self.sdk else []
        parts += ['-toolchain', self.toolchain] if self.toolchain else []
        parts += ['-jobs', self.jobs] if self.jobs else []
        parts += ['-derivedDataPath', self.derived_data_path] if self.derived_data_path else []
        parts += (['{}={}'.format(key, value) for key, value in self.arguments.items()]) if self.arguments else []
        return parts


########################################################################################################################


class XcodeBuildProduct(object):
    @classmethod
    def build_settings(cls, build_settings):
        assert isinstance(build_settings, dict)
        product = XcodeBuildProduct()
        product.build_settings = build_settings
        product.full_product_name = build_settings['FULL_PRODUCT_NAME']  # 'Example.framework'
        product.product_name = build_settings['PRODUCT_NAME']  # 'Example'
        product.executable_name = build_settings['EXECUTABLE_NAME']  # 'Example'
        product.target_build_dir = Path(build_settings['TARGET_BUILD_DIR'])  # ~/Library/Developer/Xcode/DerivedData/Example-<random>/Build/Products/<configuration>-<sdk>
        return product

    def __repr__(self):
        return 'BuildProduct({})'.format(self.__dict__)

    @classmethod
    def string(cls, string):
        lines = iter(string.splitlines())
        matches = (re.match(r'^    (.+) = (.+)$', line) for line in lines)
        matches = (match.groups() for match in matches if match)
        build_settings = dict(matches)

        product = XcodeBuildProduct.build_settings(build_settings=build_settings)
        return product

    def __init__(self):
        self.build_settings = dict()
        self.full_product_name = None
        self.product_name = None
        self.executable_name = None
        self.target_build_dir = None

    @property
    def product_path(self):
        return self.target_build_dir / self.full_product_name

    @property
    def executable_path(self):
        return self.product_path / self.executable_name

    @property
    def uuids(self):
        return uuids_from_binary(self.executable_path)

    @property
    def bcsymbolmap_paths(self):
        paths = [self.target_build_dir / (uuid + '.bcsymbolmap') for uuid in self.uuids]
        paths = [path for path in paths if path.exists()]
        return paths

    @property
    def module_paths(self):
        modules_source_path = self.product_path / "Modules/{}.swiftmodule".format(self.executable_name)
        if not modules_source_path.exists():
            return []
        return list(modules_source_path.glob("*"))


########################################################################################################################

def _parse_info(string):
    lines = iter(string.splitlines())
    targets = []
    configurations = []
    schemes = []
    default_configuration = None

    try:
        while True:
            line = six.next(lines)
            if re.match(r'^\s+Targets:$', line):
                while True:
                    line = six.next(lines)
                    match = re.match(r'        (.+)', line)
                    if not match:
                        break
                    else:
                        targets.append(match.group(1))
            if re.match(r'^\s+Build Configurations:$', line):
                while True:
                    line = six.next(lines)
                    match = re.match(r'        (.+)', line)
                    if not match:
                        break
                    else:
                        configurations.append(match.group(1))
            if re.match(r'^\s+Schemes:$', line):
                while True:
                    line = six.next(lines)
                    match = re.match(r'        (.+)', line)
                    if not match:
                        break
                    else:
                        schemes.append(match.group(1))

            match = re.match(r'^\s+If no build configuration is specified and -scheme is not passed then "(.+)" is used.', line)
            if match:
                default_configuration = match.group(1)

    except StopIteration:
        pass


    return targets, configurations, schemes, default_configuration


########################################################################################################################

def _parse_build_settings(string):
    # TODO : This is woefully inadequate

    lines = iter(string.splitlines())
    lines = (line.strip() for line in lines)

    all_build_settings = list()
    current_build_settings = dict()

    for line in lines:
        match = re.match(r'^Build settings for action (?P<action>.+) and target "?(?P<target>.+)"?:$', line)
        if match:
            if current_build_settings:
                all_build_settings.append(current_build_settings)
            current_build_settings = dict()
            next_action, next_target = (match.groupdict()['action'], match.groupdict()['target'])
            assert next_action == 'build'
            continue
        match = re.match(r'^(?P<setting>.+) = (?P<value>.+)$', line)
        if match:
            setting = match.groupdict()["setting"]
            value = match.groupdict()["value"]
            current_build_settings[setting] = value

    if current_build_settings:
        all_build_settings.append(current_build_settings)

    return dict([(build_settings['TARGET_NAME'], build_settings) for build_settings in all_build_settings if 'TARGET_NAME' in build_settings])


########################################################################################################################

def uuids_from_binary(path):
    command = ['/usr/bin/xcrun', 'dwarfdump', '--uuid', path]
    output = runner.check_run(command)
    lines = output.splitlines()
    matches = [re.match(r'^UUID: ([0-9A-F]{8}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{4}-[0-9A-F]{12}) \((.+)\) (.+)$', line) for line in lines]
    uuids = [match.group(1) for match in matches]
    return uuids



from __future__ import division, absolute_import, print_function

__all__ = ['Runner', 'runner', 'Result', 'CalledProcessError']

import logging
import subprocess
import shlex
import shelve
from subprocess import CalledProcessError

from memoize import mproperty
import six

class Result(object):
    def __init__(self):
        self.return_code = None
        self.stdout = None
        self.stderr = None


class Runner(object):
    def __init__(self, cache_path=None):
        self.cache_path = cache_path
        self.echo = False
        self.echo_directories = True

    @mproperty
    def shelf(self):
        if not self.cache_path:
            return None
        # noinspection PyBroadException
        try:
            shelf = shelve.open(str(self.cache_path))
            return shelf
        except:
            if self.cache_path.exists():
                logging.info("Resetting cache and trying again...")
                self.cache_path.unlink()
                shelf = shelve.open(str(self.cache_path))
                return self
            else:
                raise
        return shelf

    def reset(self):
        if self.cache_path.exists():
            self.shelf.close()
            self.cache_path.unlink()
            # TODO: Reopen

    def result(self, command):
        result = self.run(command)
        return result.return_code

    @staticmethod
    def convert_args(args):
        if isinstance(args, list) or isinstance(args, tuple):
            return [str(arg) for arg in args]
        elif isinstance(args, six.string_types):
            return shlex.split(args)
        else:
            return [str(args)]

    def can_run(self, args):
        args = self.convert_args(args)
        result = self.run(['/usr/bin/env', 'which', args[0]], echo=False)
        return True if result.return_code == 0 else False

    def check_run(self, *args, **kwargs):
        kwargs['check'] = True
        result = self.run(*args, **kwargs)
        return result.stdout

    # TODO: Cleanup
    check_call = check_run

    def run(self, command, cwd=None, echo=None, cache_key=None, check=False, env=None):
        args = self.convert_args(command)

        if echo or echo is None and self.echo:
            if cwd and self.echo_directories:
                logging.info('cd {}'.format(cwd))

            # TODO: Wont properly reproduce command if command is a string
            logging.info(' '.join(arg.replace(' ', '\\ ') for arg in args))

        if cache_key:
            # assert not env # TODO
            key = '{}{}'.format(cache_key, ' '.join(command))
            if key in self.shelf:
                # logger.debug('CACHE HIT: {}'.format(key))
                return_code, stdout, stderr = self.shelf[key]
                result = Result()
                result.return_code = return_code
                result.stdout = stdout
                result.stderr = stderr
                return result
            else:
                # logger.debug('CACHE MISS: {}'.format(key))
                pass

        stdout = subprocess.PIPE
        stderr = subprocess.PIPE if not check else subprocess.STDOUT

        if cwd:
            cwd = str(cwd)

        popen = subprocess.Popen(args, cwd=cwd, stdout=stdout, stderr=stderr, env=env)
        stdout, stderr = popen.communicate()

        if stdout:
            stdout = six.text_type(stdout, encoding='utf-8')
        if stderr:
            stderr = six.text_type(stderr, encoding='utf-8')

        return_code = popen.returncode

        if check and return_code != 0:
            # TODO
            if stdout:
                logging.debug(stdout)
            if stderr:
                logging.debug(stdout)
            raise CalledProcessError(return_code, command, stdout)

        if cache_key:
            key = '{}{}'.format(cache_key, ' '.join(command))
            self.shelf[key] = return_code, stdout, stderr

        result = Result()
        result.return_code = return_code
        result.stdout = stdout
        result.stderr = stderr

        return result


runner = Runner()



import os

from pathlib2 import Path
from jinja2 import Environment, PackageLoader
import click

from xcute.xcode import *
from prettytable import PrettyTable


def find_projects(xcode, path, ignore = None):
    # type: (Path, (Path) -> bool) -> XcodeProject
    for project_path in path.glob('**/*.xcodeproj'):
        if ignore and ignore(project_path):
            continue
        yield XcodeProject(xcode, project_path)

def all_build_arguments_for_project(project):
    for scheme in project.schemes:
        for target in scheme.targets:
            for platform in target.supported_platform_names:
                for configuration in ['Debug', 'Release']:
                    argument = XcodeBuildArguments(scheme = scheme.name, target = target.name, configuration=configuration, sdk=platform)
                    yield argument

def commands_for_project(project):
    options = all_build_arguments_for_project(project)

    def compare(option):
        return [project.path, option.sdk, option.scheme, option.configuration]

    options = sorted(options, key=compare)

    for option in options:
        command = 'xcodebuild -project \'{project.path}\' -scheme \'{option.scheme}\' -sdk {option.sdk} -configuration {option.configuration} build'.format(project = project, option = option)
        yield command



import os

from pathlib2 import Path
from jinja2 import Environment, PackageLoader
import click

from xcute.xcode import *
from xcute.utilities import *
from prettytable import PrettyTable

@click.group()
def xcute_cli():
    #os.chdir("/Users/schwa/Projects/SwiftIO")
    #travis()
    pass

@xcute_cli.command()
@click.option('--setting', '-s', multiple=True)
def list(setting):

    setting = [f for f in setting]

    default_xcode = Xcode.default()

    def ignore(path):
        return 'Carthage/Checkouts' in str(path)

    projects = find_projects(default_xcode, Path('.'), ignore)

    columns = ['Project', 'Scheme', 'SDK', 'Configuration'] + setting
    x = PrettyTable(columns)

    for project in projects:
        for arg in all_build_arguments_for_project(project):
            arg.target = None
            settings = project.build_settings(arg)
            settings = settings[arg.scheme]

            row = [project.path, arg.scheme, arg.sdk, arg.configuration]

            for s in setting:
                row.append(settings.get(s, ''))

            x.add_row(row)
    print(x)


@xcute_cli.command()
@click.option('--template')
def export(template):
    default_xcode = Xcode.default()

    def ignore(path):
        return 'Carthage/Checkouts' in str(path)

    projects = find_projects(default_xcode, Path('.'), ignore)
    commands = (command for project in projects for command in commands_for_project(project))

    env = Environment(loader=PackageLoader('xcute', 'templates'))
    template = env.get_template('{}.jinja2'.format(template))
    print(template.render(commands=commands, travis_xcode_version='xcode8'))

if __name__ == '__main__':
    import shlex
    import sys

    #sys.argv = shlex.split('xcute list -s SWIFT_VERSION -s ENABLE_BITCODE -s arch -s XYZ')
    sys.argv = shlex.split('xcute travis')

    xcute_cli()


from __future__ import division, absolute_import, print_function

__all__ = ['SemanticVersion']

import re
from functools import total_ordering


@total_ordering
class SemanticVersion(object):

    expression = re.compile(r'^(?:v)?(?P<major>\d+)(?:\.(?P<minor>\d+)(?:\.(?P<patch>\d+))?)?(?:-(?P<identifiers>.+))?$')

    @classmethod
    def is_semantic(cls, s):
        """
        >>> SemanticVersion.is_semantic("1.0")
        True
        >>> SemanticVersion.is_semantic("x.0")
        False
        """
        # type: (str) -> bool
        match = SemanticVersion.expression.match(s)
        return True if match else False

    def __init__(self, major, minor = 0, patch=None, identifiers=None):
        """
        >>> SemanticVersion(1, 0)
        1.0
        >>> SemanticVersion(1, 0, 0)
        1.0
        >>> SemanticVersion(1, identifiers = ['0'])
        1.0-0
        """
        self.major = major if major else 0
        self.minor = minor if minor else 0
        self.patch = patch if patch else 0
        self.identifiers = [Identifier(identifier) for identifier in identifiers] if identifiers else []

    @property
    def _components(self):
        """
        >>> SemanticVersion(1, 2, 3)._components
        [1, 2, 3, []]
        """
        # TODO: using a tuple breaks code
        #        return (self.major, self.minor, self.patch)
        return [self.major, self.minor, self.patch, self.identifiers]

    def __repr__(self):
        components = [self.major, self.minor] + ([self.patch] if self.patch else [])
        components = [str(component) for component in components]
        repr = '.'.join(components)
        if len(self.identifiers) >= 1:
            repr += '-' + '.'.join([str(identifier) for identifier in self.identifiers])
        return repr

    def __eq__(self, other):
        """
        >>> SemanticVersion.string('1') == SemanticVersion.string('1')
        True
        >>> SemanticVersion.string('1') == SemanticVersion.string('1.0')
        True
        >>> SemanticVersion.string('1') == SemanticVersion.string('1.0.0')
        True
        >>> SemanticVersion.string('1') != SemanticVersion.string('1')
        False
        """
        return self._components == other._components

    def __ne__(self, other):
        return not self.__eq__(other)

    def __lt__(self, other):
        """
        >>> SemanticVersion.string('1') < SemanticVersion.string('2')
        True
        >>> SemanticVersion.string('1') <= SemanticVersion.string('2')
        True
        >>> SemanticVersion.string('1.1') > SemanticVersion.string('1.0')
        True
        >>> SemanticVersion.string('v5.0.0-beta6') > SemanticVersion.string('v5.0.0-beta1')
        True
        >>> SemanticVersion.string('v5.0.0-beta1') > SemanticVersion.string('v5.0.0-beta6')
        False
        >>> SemanticVersion.string('v5.0.0-10') > SemanticVersion.string('v5.0.0-2')
        True
        >>> SemanticVersion.string('v5.0.0-dummy10') > SemanticVersion.string('v5.0.0-dummy2')
        False
        """
        return self._components < other._components

    def __hash__(self):
        """
        >>> hash(SemanticVersion(1, 0)) == hash(SemanticVersion(1, 0))
        True
        >>> hash(SemanticVersion(1, 0)) != hash(SemanticVersion(0, 1))
        True
        """
        return hash(self.major * 1000000) ^ hash(self.minor * 10000) ^ hash(self.patch * 100)

    @classmethod
    def from_dict(cls, d):
        if set(d.keys()).issubset({'major', 'minor', 'micro', 'releaselevel', 'serial'}):
            return SemanticVersion(major=d.get('major'), minor=d.get('minor'), patch=d.get('micro'))
        else:
            raise Exception('Invalid dict')

    @classmethod
    def string(cls, s):
        # type: (str) -> SemanticVersion
        """
        >>> SemanticVersion.string('1')
        1.0
        >>> SemanticVersion.string('1.2')
        1.2
        >>> SemanticVersion.string('1.2.3')
        1.2.3
        >>> SemanticVersion.string('garbage')
        Traceback (most recent call last):
          File "<stdin>", line 1, in ?
        Exception: "garbage" not a semantic version.
        >>> SemanticVersion.string('v5.0.0-beta6')
        5.0-beta6
        """
        match = SemanticVersion.expression.match(s)
        if not match:
            raise Exception('"{}" not a semantic version.'.format(s))
        d = match.groupdict()
        major = int(d['major']) if d['major'] else 0
        minor = int(d['minor']) if d['minor'] else 0
        patch = int(d['patch']) if d['patch'] else 0
        identifiers = [Identifier(identifier) for identifier in (d['identifiers'].split('.') if d['identifiers'] else [])]
        return SemanticVersion(major=major, minor=minor, patch=patch, identifiers=identifiers)

    @property
    def next_major(self):
        """
        >>> SemanticVersion.string('1.2').next_major
        2.0
        """
        # type: () -> SemanticVersion
        return SemanticVersion(major=self.major + 1, minor=0, patch=0)



class Identifier(object):

    def __init__(self, value):
        if isinstance(value, Identifier):
            value = value.string_value

        self.string_value = value
        self.int_value = int(value) if RepresentsInt(value) else None


    def __repr__(self):
        return self.string_value


    def __eq__(self, other):
        return self.string_value == other.string_value


    def __ne__(self, other):
        return not self.__eq__(other)


    def __lt__(self, other):
        if self.int_value is not None and other.int_value is not None:
            return self.int_value < other.int_value
        else:
            return self.string_value < other.string_value


    def __hash__(self):
        return hash(self.string_value)

INT_RE = re.compile(r"^[-]?\d+$")
def RepresentsInt(s):
    return INT_RE.match(str(s)) is not None


from distutils.core import setup

setup(
    name='xcute',
    version='0.0.2',
    packages=['xcute', 'xcute.templates'],
    url='https://github.com/schwa/xcute',
    license='MIT',
    author='schwa',
    author_email='jwight@mac.com',
    description='xcode utilitiy', # TODO: flush this out
    keywords=['build', 'xcode', 'iOS', 'cocoa', 'macOS'],
    platform="MacOS X",
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: MacOS X', 'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: MacOS :: MacOS X',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.5',
        'Topic :: Software Development :: Build Tools',
    ],
    install_requires=[
        'click',
        'click_didyoumean',
        'jinja2',
        'memoize',
        'pathlib2',
        'prettytable',
        'six',
    ],
    entry_points='''
    [console_scripts]
    xcute=xcute.main:xcute_cli
    ''', )
